@@ -1,29 +1,20 @@
 /************************************************************************
  * cmds.c -- part of rpncalc.c						*
  *									*
- * A little RPN (Reverse Polish Notation) calculator,                   *
- * rudimentary emulating a HP 28S. 					*
- * 								        *
- * rpncalc is (c) David Frey, 1993, 1994, 1995				*
- *								        * 
- * This program is free software; you can redistribute it and/or modify *
- * it under the terms of the GNU General Public License as published by *
- * the Free Software Foundation; either version 2 of the License, or    *
- * (at your option) any later version.                                  *
- *									* 
- * This program is distributed in the hope that it will be useful,      *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
- * GNU General Public License for more details.                         *
- *									* 
- * You should have received a copy of the GNU General Public License    *
- * along with this program; if not, write to the Free Software          *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
- *									* 
+ * Refer to rpncalc.c for copyrights and license permissions.           *
  ************************************************************************/
 
-/* $Id: cmds.c,v 1.2 1996/09/13 20:21:29 david Rel $
+/* $Id: cmds.c,v 1.5 1997/01/19 20:01:18 david Rel $
  * $Log: cmds.c,v $
+ * Revision 1.5  1997/01/19 20:01:18  david
+ * FreeBSD `port': Added #define EOVERFLOW ERANGE.
+ *
+ * Revision 1.4  1997/01/19 18:49:47  david
+ * Removed some casts to make gcc -Wall -W... happy.
+ *
+ * Revision 1.4  1997/01/19 18:19:23  david
+ * Removed some casts to make gcc -Wall -W... happy.
+ *
  * Revision 1.2  1996/09/13 20:21:29  david
  * lclint additions
  *
@@ -51,27 +42,24 @@ static double res;
 
 /* -------------------------------------------------------- constants */
 
-double pi(void)
-{
-  return M_PI;
-}
+double pi(void) { return M_PI; }
 
-double e(void)
-{
-  return M_E;
-}
+double e(void)  { return M_E; }
 
 /* -------------------------------------------------------- unary operators */
+#ifndef linux
+double pow10(double y) { return pow(10,y); }
 
-double chs(double f)
-{
-  return -f;
-}
+double pow2(double y)  { return pow(2,y); }
+#endif
 
-double sqr(double f)
-{
-  return f * f;
-}
+#ifdef freebsd
+#define EOVERFLOW ERANGE
+#endif
+
+double chs(double f)   { return -f; }
+
+double sqr(double f)   { return f * f; }
 
 double inv(double f)
 {
@@ -80,10 +68,7 @@ double inv(double f)
   return res;
 }
 
-double log2(double f)
-{
-  return log(f) / M_LN2;
-}
+double log2(double f)  { return log(f) / M_LN2; }
 
 double fact(double d)
 {
@@ -109,31 +94,19 @@ double fact(double d)
 
 double prec(double p)
 {
-  digits=(int)ceil(p); pushtostack=0;
+  digits=ceil(p); pushtostack=0;
   return 0.0; /* dummy value */
 }
 
-double not(double l)
-{
-  return (double)(~(long int)l);
-}
+double not(double l) { return (double)(~(long int)l); }
 
 /* ------------------------------------------------------- binary operators */
 
-double plus(double s1, double s2)
-{
-  return s1 + s2;
-}
+double plus(double s1, double s2) { return s1 + s2; }
 
-double minus(double s, double m)
-{
-  return  s-m;
-}
+double minus(double s, double m)  { return  s-m; }
 
-double multiply(double f1, double f2)
-{
-  return f1 * f2;
-}
+double multiply(double f1, double f2) { return f1 * f2; }
 
 double divide(double n, double d)
 {
@@ -142,6 +115,7 @@ double divide(double n, double d)
   return res;
 }
 
+#ifndef sunos4
 static ldiv_t quotrem;
 
 double idiv(double dn, double dd)
@@ -153,7 +127,7 @@ double idiv(double dn, double dd)
   {
     if (d != 0)
     {
-      quotrem = ldiv((long int)n,(long int)d); 
+      quotrem = ldiv(n,d); 
       res = (double)quotrem.quot;
     }
     else
@@ -167,7 +141,17 @@ double idiv(double dn, double dd)
   }	    
   return res;
 }
+#else
+double idiv(double dn, double dd)
+{
+  long int n, d;
 
+  res=(long)(n/d);
+  return res;
+}
+#endif
+
+#ifndef sunos4
 double mod(double dn, double dd)
 {
  long int n, d;
@@ -177,7 +161,7 @@ double mod(double dn, double dd)
  {
    if (d != 0)
    {
-     quotrem = ldiv((long int)n,(long int)d);
+     quotrem = ldiv(n,d);
      res = (double)quotrem.rem;  
    }
    else
@@ -191,9 +175,16 @@ double mod(double dn, double dd)
  }	    
  return res;
 }
+#else
+double mod(double dn, double dd)
+{
+ res=fmod(dn, dd);
+ return res;
+}
+#endif
 
 /* Stein's Greatest Common Divisor (GCD) algorithm */
-static long int stein(double d1, double d2)
+long int stein(double d1, double d2)
 {
   long int n1, n2;
   long int d;				 /* difference */
@@ -233,13 +224,16 @@ double gcd(double dn, double dd)
   n=(long int)dn; d=(long int)dd;
   if ((n <= LONG_MAX) && (d <= LONG_MAX))
   {
-    if (d != 0) res = (double)stein(dn,dd);
+    if (d != 0) res = stein(dn,dd);
     else      { res = HUGE_VAL; errno = ERANGE; }
   }
   else
-  {	      
-    errno = EOVERFLOW;	pushtostack = 0;
-  }	    
+  {
+#ifndef sunos4	      
+    errno = EOVERFLOW;	
+#endif
+    pushtostack = 0;
+  }
   return res;
 }
 
@@ -263,7 +257,7 @@ double sum(void)
 {
   long int i,n;
 
-  n = (long int)depth(); res = 0;
+  n = depth(); res = 0;
   for(i=1;i<=n;i++) res += pick((double)i);
   return res;
 }
@@ -272,7 +266,7 @@ double prod(void)
 {
   long int i,n;
 
-  n = (long int)depth(); res = 1;
+  n = depth(); res = 1;
   for(i=1;i<=n;i++) res *= pick((double)i);
   return res;
 }
