@@ -1,29 +1,20 @@
 /************************************************************************
  * stack.c -- part of rpncalc.c						*
  *									*
- * A little RPN (Reverse Polish Notation) calculator,                   *
- * rudimentary emulating a HP 28S. 					*
- * 								        *
- * rpncalc is (c) David Frey, 1993, 1994, 1995				*
- *								        * 
- * This program is free software; you can redistribute it and/or modify *
- * it under the terms of the GNU General Public License as published by *
- * the Free Software Foundation; either version 2 of the License, or    *
- * (at your option) any later version.                                  *
- *									* 
- * This program is distributed in the hope that it will be useful,      *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
- * GNU General Public License for more details.                         *
- *									* 
- * You should have received a copy of the GNU General Public License    *
- * along with this program; if not, write to the Free Software          *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
- *									* 
+ * Refer to rpncalc.c for copyrights and license permissions.           *
  ************************************************************************/
 
-/* $Id: stack.c,v 1.2 1996/09/13 20:21:29 david Rel $
+/* $Id: stack.c,v 1.5 1997/01/19 20:02:00 david Rel $
  * $Log: stack.c,v $
+ * Revision 1.5  1997/01/19 20:02:00  david
+ * Renamed sun_sunos[45] to sunos[45]
+ *
+ * Revision 1.4  1997/01/19 18:54:16  david
+ * Some braces rearranged.
+ *
+ * Revision 1.4  1997/01/19 18:19:23  david
+ * new command `char'.
+ *
  * Revision 1.2  1996/09/13 20:21:29  david
  * lclint additions
  *
@@ -45,16 +36,17 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
+#include <ctype.h>
+#include <locale.h>
+
+#ifdef sunos5
+#include <ieeefp.h>
+#endif
 #include <limits.h>
 
 #include "cmds.h"
 #include "utils.h"
-
-struct elm
-{
-  double value;
-  struct elm *prev, *next;
-};
+#include "stack.h"
 
 struct elm *anchor=NULL;
 
@@ -86,7 +78,10 @@ double pop(void)
   if (anchor != NULL)
   {
     value=anchor->value; anchor=anchor->next; 
-    if (anchor != NULL) anchor->prev=NULL;
+    if (anchor != NULL) {
+	if (anchor->prev != NULL) free(anchor->prev);
+    	anchor->prev=NULL;
+    }
   }
   else
   {
@@ -224,7 +219,7 @@ double roll(void)
  * Infinite results and NaN (not a number) will be displayed by using
  * isinf() and isnan().
  */
-double showstack(void)
+double showstack()
 {
   struct elm *p;
   double value;
@@ -236,33 +231,50 @@ double showstack(void)
     while (p->next != NULL) { p=p->next; i++; }
     while (p != NULL)
     {
+      printf("%2i: ", i);
       value=p->value;  
       switch (base)
       {
-        case HEX: printf("%2i: 0x%lx\n", i, (unsigned long int)value); break;
-        case OCT: printf("%2i: 0%lo\n",  i, (signed long int)value); break;
+        case HEX:  printf("0x%lx", (unsigned long int)value); break;
+        case OCT:  printf("0%lo",  (signed long int)value); break;
+	case CHAR: {
+			unsigned long int  v=floor(value);
+			unsigned char c=(unsigned char)v;
+
+			printf("%03ld 0%03lo 0x%02lx ",v,v,v);
+			if (isprint(c)) printf("'%c'",c);
+			else if (iscntrl(c)) {
+			  if (c < 32)        printf("^%c", c+64);
+			  else if (c == 127) printf("^?");
+			  else               printf("M-^%c", c-64);
+			}
+			break;
+		   }
         case DEC:
         default:
-	          if (isinf(value)) 
-		  {
-	            printf("%2i: %cinfinity\n", i, 
+#ifdef sunos5
+		  if (!finite(value)) {
+#else
+	          if (isinf(value)) {
+#endif
+	            printf("%cinfinity",
+#ifdef sunos5
+			   finite(value) == 0 ? '+' : '-');
+#else
 			   isinf(value) == 1 ? '+' : '-');
+#endif
 		  }
-	          else if (isnan(value))
-	          {
-		    printf("%2i: NaN\n", i);
-		  }
-	          else if ((floor(value) == value) && ((long int)value <= LONG_MAX))
-		  {
-		    printf("%2i: %.0f\n", i, value);
-		  }
-	          else
-		  {
-		    printf("%2i: %.*g\n", i, (digits+1), value);
+	          else if (isnan(value)) { printf("NaN"); }
+	          else if ((floor(value) == value) && 
+			  ((long int)value <= LONG_MAX)) {
+		    printf("%.0f",value);
+		  } else {
+		    printf("%.*g", (digits+1), value);
 		  }
 	          break;
 	  
       }
+      printf("\n");
       p=p->prev; i--;
     }  
   }
@@ -286,3 +298,9 @@ double setoct(void)
   base = OCT;
   return 0.0; /* dummy value */
 }
+
+double setchar(void)
+{
+  base = CHAR;
+  return 0.0; /* dummy value */
+}
