@@ -1,29 +1,17 @@
 /************************************************************************
  * utils.c -- various utilities for rpncalc				*
  *									*
- * A little RPN (Reverse Polish Notation) calculator,                   *
- * rudimentary emulating a HP 28S. 					*
- * 								        *
- * rpncalc is (c) David Frey, 1993, 1994, 1995				*
- *								        * 
- * This program is free software; you can redistribute it and/or modify *
- * it under the terms of the GNU General Public License as published by *
- * the Free Software Foundation; either version 2 of the License, or    *
- * (at your option) any later version.                                  *
- *									* 
- * This program is distributed in the hope that it will be useful,      *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
- * GNU General Public License for more details.                         *
- *									* 
- * You should have received a copy of the GNU General Public License    *
- * along with this program; if not, write to the Free Software          *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
- *									* 
+ * Refer to rpncalc.c for copyrights and license permissions.           *
  ************************************************************************/
 
-/* $Id: utils.c,v 1.2 1996/09/13 20:21:29 david Rel $
+/* $Id: utils.c,v 1.4 1997/01/19 18:58:58 david Rel $
  * $Log: utils.c,v $
+ * Revision 1.4  1997/01/19 18:58:58  david
+ * Changed dupstr(char *) to dupstr(const char *)
+ *
+ * Revision 1.4  1997/01/19 18:19:23  david
+ * Changed dupstr(char *) to dupstr(const char *)
+ *
  * Revision 1.2  1996/09/13 20:21:29  david
  * lclint additions
  *
@@ -40,6 +28,9 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <strings.h>
+
+#include "utils.h"
 
 #ifdef HAVE_READLINE
 #include <readline/readline.h>
@@ -74,11 +65,11 @@
   return ptr;
 }
 
-char *dupstr(char *s)
+char *dupstr(const char *s)
 {
   char *r;
 
-  r = xmalloc (strlen (s) + 1);
+  r = (char *)xmalloc (strlen (s) + 1);
   strcpy (r, s);
   return (r);
 }
@@ -87,21 +78,21 @@ char *dupstr(char *s)
 /* Read a line of text without imposing an length limit.
    If we readline support is compiled in, use the readline function. */
 {
-  unsigned char *line=NULL;
+  char *line=NULL;
 #ifndef HAVE_READLINE
   int c;
   int i;
 #endif
 
 #ifdef HAVE_READLINE
-  line=(unsigned char*)readline(""); 
+  line=(char*)readline(""); 
 #else
-  i=0; line=(unsigned char *)xmalloc((size_t)(LINEFRAGMENT+1));
+  i=0; line=(char *)xmalloc((size_t)(LINEFRAGMENT+1));
   while (((c = getc(stdin)) != EOF) && (c != '\n') && (c != '\0'))
   {
    if ((i > LINEFRAGMENT) &&  (i % LINEFRAGMENT) == 0) 
-     line=(unsigned char *)xrealloc(line,(size_t)(LINEFRAGMENT+1));
-   line[i]=(unsigned char)c; i++;
+     line=(char *)xrealloc(line,(size_t)(LINEFRAGMENT+1));
+   line[i]=(char)c; i++;
   }
   if ((c != EOF) && (c != '\0')) line[i]='\0';
   else
